@@ -85,6 +85,15 @@ class NoStreamError(GStreamerError):
     def __init__(self):
         super(NoStreamError, self).__init__('no audio streams found')
 
+class IncompleteGStreamerError(GStreamerError):
+    """Raised when necessary components of GStreamer (namely, the
+    principal plugin packages) are missing.
+    """
+    def __init__(self):
+        super(IncompleteGStreamerError, self).__init__(
+            'missing GStreamer base plugins'
+        )
+
 
 # Managing the Gobject main loop thread.
 
@@ -141,9 +150,14 @@ class GstAudioFile(object):
         
         # Set up the Gstreamer pipeline.
         self.pipeline = gst.Pipeline()
-        self.dec = gst.element_factory_make("uridecodebin")
-        self.conv = gst.element_factory_make("audioconvert")
-        self.sink = gst.element_factory_make('appsink')
+        try:
+            self.dec = gst.element_factory_make("uridecodebin")
+            self.conv = gst.element_factory_make("audioconvert")
+            self.sink = gst.element_factory_make("appsink")
+        except gst.ElementNotFoundError:
+            # uridecodebin, audioconvert, or appsink is missing. We need
+            # gst-plugins-base.
+            raise IncompleteGStreamerError()
         
         # Register for bus signals.
         bus = self.pipeline.get_bus()
@@ -152,7 +166,7 @@ class GstAudioFile(object):
         bus.connect("message::error", self._message)
         
         # Configure the input.
-        uri = 'file://' + urllib.quote(path)
+        uri = 'file://' + urllib.quote(os.path.abspath(path))
         self.dec.set_property("uri", uri)
         # The callback to connect the input.
         self.dec.connect("pad-added", self._pad_added)
@@ -202,6 +216,7 @@ class GstAudioFile(object):
         
         # Return as soon as the stream is ready!
         self.running = True
+        self.got_caps = False
         self.pipeline.set_state(gst.STATE_PLAYING)
         self.ready_sem.acquire()
         if self.read_exc:
@@ -216,6 +231,7 @@ class GstAudioFile(object):
         # The sink has started to receive data, so the stream is ready.
         # This also is our opportunity to read information about the
         # stream.
+        self.got_caps = True
         info = pad.get_negotiated_caps()[0]
         
         # Stream attributes.
@@ -279,6 +295,12 @@ class GstAudioFile(object):
             if message.type == gst.MESSAGE_EOS:
                 # The file is done. Tell the consumer thread.
                 self.queue.put(SENTINEL)
+                if not self.got_caps:
+                    # If the stream ends before _notify_caps was called, this
+                    # is an invalid file.
+                    self.read_exc = NoStreamError()
+                    self.ready_sem.release()
+
             elif message.type == gst.MESSAGE_ERROR:
                 gerror, debug = message.parse_error()
                 if 'not-linked' in debug:
